<?php

namespace Drupal\snippet_manager;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Utility\Timer;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\snippet_manager\Entity\Snippet;

/**
 * View builder for snippets.
 */
class SnippetViewBuilder extends EntityViewBuilder {

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $snippet, $view_mode = 'full', $langcode = NULL) {

    switch ($view_mode) {
      // Entity display repository provides 'default' option when asked for
      // available display modes.
      // See \Drupal\Entity\EntityDisplayRepository::getDisplayModeOptions().
      case 'default':
      case 'full':
      case NULL:
        $build['snippet'] = [
          '#cache' => [
            'keys' => ['snippet', $snippet->id()],
            'tags' => Cache::mergeTags($this->getCacheTags(), $snippet->getCacheTags()),
          ],
          '#lazy_builder' => [static::class . '::lazyBuilder', [$snippet->id()]],
        ];
        break;

      case 'source':
        $build = $this->viewSource($snippet);
        break;

      default:
        throw new \InvalidArgumentException('Unsupported view mode.');
    }

    return $build;
  }

  /**
   * Builds the render array for the source view mode.
   *
   * @param \Drupal\Core\Entity\EntityInterface $snippet
   *   A snippet being viewed.
   *
   * @return array
   *   Render array to display snippet HTML source code.
   */
  protected function viewSource(EntityInterface $snippet) {

    $build['snippet']['content']['#type'] = 'textarea';
    Timer::start('snippet');
    $value = $this->view($snippet);
    // Render the snippet immediately to calculate render time.
    $build['snippet']['content']['#value'] = (string) render($value);
    $render_time = Timer::read('snippet');
    $build['snippet']['content']['#attributes']['class'][] = 'snippet-html-source';

    $build['render_time_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => 'snippet-render-time'],
    ];

    $build['render_time_wrapper']['render_time'] = [
      '#markup' => $this->t('Render time: %time ms', ['%time' => $render_time]),
    ];

    $build['#attached']['library'][] = 'snippet_manager/editor';

    return $build;
  }

  /**
   * Lazy builder callback; builds a #pre_render-able snippet.
   *
   * @param string $snippet_id
   *   Snippet ID.
   *
   * @return array
   *   A render array with a #pre_render callback to render the snippet.
   */
  public static function lazyBuilder($snippet_id) {
    return static::doBuild(Snippet::load($snippet_id));
  }

  /**
   * Builds render array.
   *
   * @param \Drupal\snippet_manager\SnippetInterface $snippet
   *   A snippet.
   *
   * @return array
   *   Snippet content as render array.
   */
  public static function doBuild(SnippetInterface $snippet) {

    /** @var \Drupal\snippet_manager\SnippetVariablePluginManager $variable_manager */
    $variable_manager = \Drupal::service('plugin.manager.snippet_variable');

    $module_handler = \Drupal::moduleHandler();
    // The context is an array of processed Twig variables.
    $context = $module_handler->invokeAll('snippet_context', [$snippet]);
    foreach ($snippet->get('variables') as $variable_name => $variable) {
      try {
        $plugin = $variable_manager->createInstance(
          $variable['plugin_id'],
          $variable['configuration'],
          $snippet,
          $variable_name
        );
        $context[$variable_name] = $plugin->build();
      }
      catch (PluginNotFoundException $exception) {
        $context[$variable_name] = '';
      }
    }
    $module_handler->alter('snippet_context', $context, $snippet);

    $template = $snippet->get('template');

    $build['snippet'] = [
      '#type' => 'inline_template',
      '#template' => check_markup($template['value'], $template['format']),
      '#context' => $context,
    ];

    if ($snippet->get('css')['status'] || $snippet->get('js')['status']) {
      $build['snippet']['#attached']['library'][] = 'snippet_manager/snippet_' . $snippet->id();
    }

    return $build;
  }

}
